/**
 * @file
 * VuoList_VuoPoint2d C type definition.
 *
 * @copyright Copyright © 2012–2017 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#pragma once

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoPoint2d VuoList_VuoPoint2d
 * A list of @ref VuoPoint2d elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoPoint2d_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoPoint2d elements.
 */
typedef const struct VuoList_VuoPoint2d_struct { void *l; } * VuoList_VuoPoint2d;
/// @}
#endif

/**
 * Creates a new list of @ref VuoPoint2d elements.
 */
VuoList_VuoPoint2d VuoListCreate_VuoPoint2d(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoPoint2d to quickly initialize a large list.
 */
VuoList_VuoPoint2d VuoListCreateWithCount_VuoPoint2d(const unsigned long count, const VuoPoint2d value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoPoint2d VuoListCopy_VuoPoint2d(const VuoList_VuoPoint2d list);

/**
 * Returns the @ref VuoPoint2d at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoPoint2d VuoListGetValue_VuoPoint2d(const VuoList_VuoPoint2d list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoPoint2d *VuoListGetData_VuoPoint2d(const VuoList_VuoPoint2d list);

/**
 * Changes the @ref VuoPoint2d at @c index.
 * Index values start at 1.
 *
 * If `expandListIfNeeded` is false: If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 *
 * If `expandListIfNeeded` is true: If the list has less than `index` elements, it is expanded to accommodate the specified `index`.
 * If `index` is 0, the list is resized to contain 1 item set to `value`.
 */
void VuoListSetValue_VuoPoint2d(const VuoList_VuoPoint2d list, const VuoPoint2d value, const unsigned long index, bool expandListIfNeeded);

/**
 * Inserts the `VuoPoint2d` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoPoint2d(const VuoList_VuoPoint2d list, const VuoPoint2d value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoPoint2d(VuoList_VuoPoint2d list, const VuoPoint2d value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoPoint2d(VuoList_VuoPoint2d list, const VuoPoint2d value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoPoint2d(VuoList_VuoPoint2d list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoPoint2d_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoPoint2d(VuoList_VuoPoint2d list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoPoint2d_areEqual(const VuoList_VuoPoint2d a, const VuoList_VuoPoint2d b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoPoint2d_isLessThan(const VuoList_VuoPoint2d a, const VuoList_VuoPoint2d b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoPoint2d(VuoList_VuoPoint2d list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoPoint2d(VuoList_VuoPoint2d list);

/**
 * Returns a new list containing items from the original list in the range specified by `startIndex` and `itemCount`.
 *
 * Items in the new list are retained (not copied) from the original list.
 */
VuoList_VuoPoint2d VuoListSubset_VuoPoint2d(VuoList_VuoPoint2d list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoPoint2d(VuoList_VuoPoint2d list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoPoint2d(VuoList_VuoPoint2d list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoPoint2d(VuoList_VuoPoint2d list);

/**
 * Removes the `VuoPoint2d` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoPoint2d(VuoList_VuoPoint2d list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoPoint2d(const VuoList_VuoPoint2d list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoPoint2d VuoList_VuoPoint2d_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoPoint2d_getJson(const VuoList_VuoPoint2d value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoPoint2d_getSummary(const VuoList_VuoPoint2d value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoPoint2d VuoList_VuoPoint2d_makeFromString(const char *str);
char * VuoList_VuoPoint2d_getString(const VuoList_VuoPoint2d value);
/// @}

/**
 * @}
 */

#ifdef __cplusplus
}
#endif
